package ru.ifmo.vizi.RadixSort;

import ru.ifmo.vizi.base.ui.*;
import ru.ifmo.vizi.base.*;
import ru.ifmo.vizi.base.widgets.Rect;
import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;
import java.util.Stack;

/**
 * Radix Sort applet.
 *
 * @author  Georgiy Korneev
 * @version $Id: RadixSortVisualizer.java,v 1.4 2004/01/23 14:40:04 geo Exp $
 */
public final class RadixSortVisualizer extends Base {
    /**
     * Radix Sort automata instance.
     */
    private final RadixSort auto;

    /**
     * Radix Sort automata data.
     */
    private final RadixSort.Data data;

    /**
     * Cells with array elements.
     * Vector of {@link Rect}.
     */
    private final Stack cells;
    private final Stack cells_b;
    private final Stack cells_c;

    /**
     * Number of elements in array.
     */
    private final SpinPanel elements;

    /**
     * Maximal array value.
     */
    private final int maxValue;

    /**
     * Maximal array value string.
     */
    private final String maxValueString;


    /**
     * Array shape style set.
     */
    private final ShapeStyle[] styleSet;

    /**
     * Save/load dialog.
     */
    private SaveLoadDialog saveLoadDialog;

    /**
     * Creates a new RadixSort  visualizer.
     *
     * @param parameters visualizer parameters.
     */
    public RadixSortVisualizer(VisualizerParameters parameters) {
        super(parameters);
        auto = new RadixSort(locale);
        data = auto.d;
        data.visualizer = this;
        cells = new Stack();
        cells_b = new Stack();
        cells_c = new Stack();

        styleSet = ShapeStyle.loadStyleSet(config, "array");

        elements = new SpinPanel(config, "elements") {
            protected void click(double value) {
                setArraySize(getIntValue());
            }
        };

        maxValue = config.getInteger("max-value");
        maxValueString = config.getParameter("max-value-string", Integer.toString(maxValue));
        setArraySize(elements.getIntValue());

        createInterface(auto);
    }

    /**
     * This method creates panel with visualizer controls.
     *
     * @return controls pane.
     */
    public Component createControlsPane() {
        Panel panel = new Panel(new BorderLayout());

        panel.add(new AutoControlsPane(config, auto, forefather, false), BorderLayout.CENTER);

        Panel bottomPanel = new Panel();
        bottomPanel.add(new HintedButton(config, "button-random"){
            protected void click() {
                randomize();
            }
        });
        if (config.getBoolean("button-ShowSaveLoad")) { 
            bottomPanel.add(new HintedButton(config, "button-SaveLoad") {
                protected void click() {
                    saveLoadDialog.center();
                    StringBuffer buffer = new StringBuffer();
                    int[] a = auto.d.a;
                    buffer.append("/* ").append(
                        I18n.message(
                            config.getParameter("ArrayLengthComment"), 
                            new Double(elements.getMinValue()),
                            new Double(elements.getMaxValue())
                        )
                    ).append(" */\n");

                    buffer.append("ArrayLength = ").append(a.length).append("\n");

                    buffer.append("/* ").append(
                        I18n.message(
                            config.getParameter("ElementsComment"), 
                            new Integer(0),
                            new Integer(maxValue)
                        )
                    ).append(" */\n");

                    buffer.append("Elements = ");
                    for (int i = 0; i < a.length; i++) {
                        buffer.append(a[i]).append(" ");
                    }

                    buffer.append("\n/* ").append(
                        config.getParameter("StepComment")
                    ).append(" */\n");
                    buffer.append("Step = ").append(auto.getStep());
                    saveLoadDialog.show(buffer.toString());
                }
            });
        }
        bottomPanel.add(elements);
        panel.add(bottomPanel, BorderLayout.SOUTH);

        saveLoadDialog = new SaveLoadDialog(config, forefather) {
            public boolean load(String text) throws Exception {
                SmartTokenizer tokenizer = new SmartTokenizer(text, config);
                tokenizer.expect("ArrayLength");
                tokenizer.expect("=");
                int[] a = new int[tokenizer.nextInt(
                    (int) elements.getMinValue(), 
                    (int) elements.getMaxValue()
                )];

                tokenizer.expect("Elements");
                tokenizer.expect("=");
                for (int i = 0; i < a.length; i++) {
                    a[i] = tokenizer.nextInt(0, maxValue);
                }

                tokenizer.expect("Step");
                tokenizer.expect("=");
                int step = tokenizer.nextInt();
                tokenizer.expectEOF();

                auto.d.a = a;
                rewind(step);

                return true;
            }
        };

        return panel;
    }

    /**
     * Adjusts array size to match current model size.
     */
    private void adjustArraySize() {
        int size = auto.d.a.length;
        while (cells.size() < size) {
            Rect rect = new Rect(styleSet);
            cells.push(rect);
            clientPane.add(rect);
        }
        while (cells.size() > size) {
            clientPane.remove((Component) cells.pop());
        }

        while (cells_b.size() < size) {
            Rect rect = new Rect(styleSet);
            cells_b.push(rect);
            clientPane.add(rect);
        }
        while (cells_b.size() > size) {
            clientPane.remove((Component) cells_b.pop());
        }

        while (cells_c.size() < 10) {
            Rect rect = new Rect(styleSet);
            cells_c.push(rect);
            clientPane.add(rect);
        }
        while (cells_c.size() > 10) {
            clientPane.remove((Component) cells_c.pop());
        }


        clientPane.doLayout();
        elements.setValue(data.a.length);
    }

    /**
     * Sets new array size.
     *
     * @param size new array size.
     */
    private void setArraySize(int size) {
        auto.d.a = new int[size];
        randomize();
        adjustArraySize();
    }

    /**
     * Randomizes array values.
     */
    private void randomize() {
        for (int i = 0; i < data.a.length; i++) {
            data.a[i] = (int) (Math.random() * maxValue) + 1;
        }
        rewind(0);
    }

    /**
     * Rewinds algorithm to the specified step.
     *
     * @param step spet of the algorith to rewind to.
     */
    private void rewind(int step) {
        adjustArraySize();
        auto.toStart();

        updateArray(-1, -1, -1);
        while (!auto.isAtEnd() && auto.getStep() < step) {
            auto.stepForward(0);
        }
    }

    /**
     * Updates array view.
     *
     * @param activeCell current active cell.
     * @param activeStyle style of active cell.
     */
    public void updateArray(int a_index, int counter_index, int res_index) {


        for (int i = 0; i < data.a.length; i++) {
            Rect rect = (Rect) cells.elementAt(i);
            StringBuffer s = new StringBuffer();
            int alength = Integer.toString(data.a[i]).length();
            for(int l = 0; l < 4 - alength; l++){
                s.append("0");
            }
            s.append(Integer.toString(data.a[i]));
            rect.setMessage(s.toString());
            rect.setStyle(i == a_index ? 1 : 0);
        }

        for (int i = 0; i < data.res.length; i++) {
            Rect rect = (Rect) cells_b.elementAt(i);
            if(data.res[i] > 0) {
                StringBuffer s = new StringBuffer();
                int blength = Integer.toString(data.res[i]).length();
                for(int l = 0; l < 4 - blength; l++){
                s.append("0");
                }
                s.append(Integer.toString(data.res[i]));
                rect.setMessage(s.toString());
                rect.setStyle(i == res_index ? 1 : 0);
            }else {
                rect.setMessage("");
                rect.setStyle(0);
            }
       
        }

        for (int i = 0; i < data.counter.length; i++) {
            Rect rect = (Rect) cells_c.elementAt(i);
            StringBuffer s = new StringBuffer();
            s.append(Integer.toString(data.counter[i]));
            rect.setMessage(s.toString());
            rect.setStyle(i == counter_index ? 1 : 0);
       
        }

        update(true);
    }

    /**
     * Invoked when client pane shoud be layouted.
     *
     * @param clientWidth client pane width.
     * @param clientHeight client pane height.
     */
    protected void layoutClientPane(int clientWidth, int clientHeight) {
        int n = cells.size();


        double width = (clientWidth - 10.0) / n;
        double height = Math.min(width, (clientHeight) * 10.0 / 30.0);
        double y = height / 10;
        double x = (clientWidth - width * n) / 2;

        double counter_width = (clientWidth - 10) /10;
        double counter_height = Math.min(counter_width, (clientHeight) * 10.0 / 30.0);

        for (int i = 0; i < n; i++) {
            Rect rect = (Rect) cells.elementAt(i);
            rect.setBounds((int)Math.round(x + i * width), (int)Math.round(y), (int)Math.round(width + 1), (int)Math.round(height + 1));
            rect.adjustFontSize(maxValueString);
        }

        for (int i = 0; i < n; i++) {
            Rect rect = (Rect) cells_b.elementAt(i);
            rect.setBounds((int)Math.round(x + i * width), (int)Math.round(y + height + counter_height + 10), (int)Math.round(width + 1), (int)Math.round(height + 1));
            rect.adjustFontSize(maxValueString);
        }

        for (int i = 0; i < 10; i++) {
            Rect rect = (Rect) cells_c.elementAt(i);
            rect.setBounds((int)Math.round(x + i * counter_width), (int)Math.round(y + height + 5), (int)Math.round(counter_width + 1), (int)Math.round(counter_height + 1));
            rect.adjustFontSize(maxValueString);
        }

    }

}

